import uuid

from sqlalchemy import Column, ForeignKey, Unicode, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship

from .meta import Base


class Contact(Base):
    __tablename__ = "contacts"

    id = Column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), default=uuid.uuid4
    )
    office_id = Column("office_id", UUID(as_uuid=True), ForeignKey("offices.id"), nullable=False)
    department_id = Column("department_id", UUID(as_uuid=True), ForeignKey("departments.id"), nullable=False)
    name = Column("name", Unicode(255), nullable=False)
    mobile = Column("mobile", Unicode(255), nullable=False, unique=True)
    landline = Column("landline", Unicode(255), nullable=False, unique=True)
    email = Column("email", Unicode(255), nullable=True)
    address = Column("address", Unicode(255), nullable=True)

    office = relationship("Office", backref="contacts")
    department = relationship("Department", backref="contacts")

    @property
    def __name__(self):
        return self.name

    def __init__(
        self,
        name=None,
        mobile=None,
        landline=None,
        address=None,
        email=None,
        office_id=None,
        department_id=None,
        id_=None,
    ):
        self.name = name
        self.mobile = mobile
        self.landline = landline
        self.address = address
        self.email = email
        self.office_id = office_id
        self.department_id = department_id
        self.id = id_
