import uuid

from fastapi import APIRouter, Depends, Security
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.master import DbSetting
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.get("/{id_}")
def get(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user),
):
    data = db.query(DbSetting).filter(DbSetting.name == id_).first()
    return {"id": data.id, "name": data.name, "details": data.data}
