import uuid

from typing import Optional

from pydantic import BaseModel, Field

from . import to_camel
from .department import DepartmentLink
from .office import OfficeLink


class ContactIn(BaseModel):
    name: str = Field(..., min_length=1)
    mobile: str
    landline: str
    address: str
    email: str
    office: OfficeLink
    department: DepartmentLink

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class Contact(ContactIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class ContactBlank(ContactIn):
    name: str
    office: Optional[OfficeLink]
    department: Optional[DepartmentLink]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class ContactLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: Optional[str]

    class Config:
        fields = {"id_": "id"}
