import uuid

from typing import Optional

from pydantic import BaseModel, Field

from . import to_camel
from .department import DepartmentLink


class OfficeIn(BaseModel):
    name: str = Field(..., min_length=1)
    address: str
    email: str
    department: Optional[DepartmentLink]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class Office(OfficeIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class OfficeBlank(OfficeIn):
    name: str
    department: Optional[DepartmentLink]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class OfficeLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: Optional[str]

    class Config:
        fields = {"id_": "id"}
