import uuid

from typing import List

from pydantic import BaseModel, Field

from .permission import PermissionItem


class RoleIn(BaseModel):
    name: str = Field(..., min_length=1)
    permissions: List[PermissionItem]

    class Config:
        anystr_strip_whitespace = True


class Role(RoleIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        fields = {"id_": "id"}


class RoleBlank(RoleIn):
    name: str

    class Config:
        anystr_strip_whitespace = True


class RoleList(BaseModel):
    id_: uuid.UUID
    name: str
    permissions: List[str]

    class Config:
        fields = {"id_": "id"}
        anystr_strip_whitespace = True


class RoleItem(BaseModel):
    id_: uuid.UUID
    name: str
    enabled: bool

    class Config:
        fields = {"id_": "id"}
