import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Act } from '../core/act';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/acts';
const serviceName = 'ActService';

@Injectable({
  providedIn: 'root',
})
export class ActService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Act> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Act>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Act>;
  }

  list(): Observable<Act[]> {
    return this.http
      .get<Act[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Act[]>;
  }

  save(act: Act): Observable<Act> {
    return this.http
      .post<Act>(url, act, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Act>;
  }

  update(act: Act): Observable<Act> {
    return this.http
      .put<Act>(`${url}/${act.id}`, act, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Act>;
  }

  saveOrUpdate(act: Act): Observable<Act> {
    if (!act.id) {
      return this.save(act);
    }
    return this.update(act);
  }

  delete(id: string): Observable<Act> {
    return this.http
      .delete<Act>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Act>;
  }
}
