import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';

import { Advocate } from '../../core/advocate';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { AdvocateService } from '../advocate.service';

@Component({
  selector: 'app-advocate-detail',
  templateUrl: './advocate-detail.component.html',
  styleUrls: ['./advocate-detail.component.css'],
})
export class AdvocateDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup;
  item: Advocate = new Advocate();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private fb: FormBuilder,
    private toaster: ToasterService,
    private ser: AdvocateService,
  ) {
    // Create form
    this.form = this.fb.group({
      name: '',
      mobile: '',
      landline: '',
      email: '',
      address: '',
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: Advocate };
      this.showItem(data.item);
    });
  }

  showItem(item: Advocate) {
    this.item = item;
    this.form.setValue({
      name: this.item.name || '',
      mobile: this.item.mobile || '',
      landline: this.item.landline || '',
      email: this.item.email || '',
      address: this.item.address || '',
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/advocates');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/advocates');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Advocate?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): Advocate {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.mobile = formModel.mobile;
    this.item.landline = formModel.landline;
    this.item.email = formModel.email;
    this.item.address = formModel.address;
    return this.item;
  }
}
