import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Advocate } from '../core/advocate';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/advocates';
const serviceName = 'AdvocateService';

@Injectable({
  providedIn: 'root',
})
export class AdvocateService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Advocate> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Advocate>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Advocate>;
  }

  list(): Observable<Advocate[]> {
    return this.http
      .get<Advocate[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Advocate[]>;
  }

  save(advocate: Advocate): Observable<Advocate> {
    return this.http
      .post<Advocate>(url, advocate, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Advocate>;
  }

  update(advocate: Advocate): Observable<Advocate> {
    return this.http
      .put<Advocate>(`${url}/${advocate.id}`, advocate, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Advocate>;
  }

  saveOrUpdate(advocate: Advocate): Observable<Advocate> {
    if (!advocate.id) {
      return this.save(advocate);
    }
    return this.update(advocate);
  }

  delete(id: string): Observable<Advocate> {
    return this.http
      .delete<Advocate>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Advocate>;
  }
}
