import { Injectable } from '@angular/core';
import { Router, CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot } from '@angular/router';

import { ToasterService } from '../core/toaster.service';

import { AuthService } from './auth.service';

@Injectable({ providedIn: 'root' })
export class AuthGuard implements CanActivate {
  constructor(
    private router: Router,
    private authService: AuthService,
    private toaster: ToasterService,
  ) {}

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot) {
    const { user } = this.authService;
    const permission =
      route.data.permission === undefined
        ? route.data.permission
        : route.data.permission.replace(/ /g, '-').toLowerCase();
    if (!user) {
      // not logged in so redirect to login page with the return url
      this.router.navigate(['/login'], { queryParams: { returnUrl: state.url } });
      return false;
    }
    if (permission !== undefined && !this.authService.allowed(permission)) {
      this.toaster.show('Error', 'You do not have the permission to access this area.');
      return false;
    }
    // logged in so return true
    return true;
  }
}
