import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { CaseType } from '../core/case-type';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/case-types';
const serviceName = 'CaseTypeService';

@Injectable({
  providedIn: 'root',
})
export class CaseTypeService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<CaseType> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<CaseType>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<CaseType>;
  }

  list(): Observable<CaseType[]> {
    return this.http
      .get<CaseType[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<CaseType[]>;
  }

  save(caseType: CaseType): Observable<CaseType> {
    return this.http
      .post<CaseType>(url, caseType, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<CaseType>;
  }

  update(caseType: CaseType): Observable<CaseType> {
    return this.http
      .put<CaseType>(`${url}/${caseType.id}`, caseType, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<CaseType>;
  }

  saveOrUpdate(caseType: CaseType): Observable<CaseType> {
    if (!caseType.id) {
      return this.save(caseType);
    }
    return this.update(caseType);
  }

  delete(id: string): Observable<CaseType> {
    return this.http
      .delete<CaseType>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<CaseType>;
  }
}
