import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { ActListResolver } from '../acts/act-list-resolver.service';
import { AuthGuard } from '../auth/auth-guard.service';
import { CaseSourceListResolver } from '../case-sources/case-source-list-resolver.service';
import { CaseTypeListResolver } from '../case-types/case-type-list-resolver.service';
import { Case } from '../core/case';
import { CaseType } from '../core/case-type';
import { Court } from '../core/court';
import { Department } from '../core/department';
import { CourtStatusListResolver } from '../court-statuses/court-status-list-resolver.service';
import { CourtListResolver } from '../courts/court-list-resolver.service';
import { DepartmentListResolver } from '../departments/department-list-resolver.service';
import { NatureListResolver } from '../natures/nature-list-resolver.service';
import { OfficeStatusListResolver } from '../office-statuses/office-status-list-resolver.service';

import { CaseDetailComponent } from './case-detail/case-detail.component';
import { CaseListResolver } from './case-list-resolver.service';
import { CaseListComponent } from './case-list/case-list.component';
import { CaseResolver } from './case-resolver.service';

const advocatesRoutes: Routes = [
  {
    path: '',
    component: CaseListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Cases',
    },
    resolve: {
      caseSources: CaseSourceListResolver,
      // list: CaseListResolver,
    },
  },
  {
    path: 'new',
    component: CaseDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Cases',
    },
    resolve: {
      item: CaseResolver,
      caseSources: CaseSourceListResolver,
      caseTypes: CaseTypeListResolver,
      courts: CourtListResolver,
      departments: DepartmentListResolver,
      courtStatuses: CourtStatusListResolver,
      officeStatuses: OfficeStatusListResolver,
      acts: ActListResolver,
      natures: NatureListResolver,
    },
  },
  {
    path: ':id',
    component: CaseDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Cases',
    },
    resolve: {
      item: CaseResolver,
      caseSources: CaseSourceListResolver,
      caseTypes: CaseTypeListResolver,
      courts: CourtListResolver,
      departments: DepartmentListResolver,
      courtStatuses: CourtStatusListResolver,
      officeStatuses: OfficeStatusListResolver,
      acts: ActListResolver,
      natures: NatureListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(advocatesRoutes)],
  exports: [RouterModule],
  providers: [CaseListResolver, CaseResolver],
})
export class CasesRoutingModule {}
