import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Contact } from '../core/contact';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/contacts';
const serviceName = 'ContactService';

@Injectable({
  providedIn: 'root',
})
export class ContactService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Contact> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Contact>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Contact>;
  }

  list(): Observable<Contact[]> {
    return this.http
      .get<Contact[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Contact[]>;
  }

  save(contact: Contact): Observable<Contact> {
    return this.http
      .post<Contact>(url, contact, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Contact>;
  }

  update(contact: Contact): Observable<Contact> {
    return this.http
      .put<Contact>(`${url}/${contact.id}`, contact, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Contact>;
  }

  saveOrUpdate(contact: Contact): Observable<Contact> {
    if (!contact.id) {
      return this.save(contact);
    }
    return this.update(contact);
  }

  delete(id: string): Observable<Contact> {
    return this.http
      .delete<Contact>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Contact>;
  }
}
