import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { DepartmentListResolver } from '../departments/department-list-resolver.service';

import { ContactDetailComponent } from './contact-detail/contact-detail.component';
import { ContactListResolver } from './contact-list-resolver.service';
import { ContactListComponent } from './contact-list/contact-list.component';
import { ContactResolver } from './contact-resolver.service';

const advocatesRoutes: Routes = [
  {
    path: '',
    component: ContactListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Contacts',
    },
    resolve: {
      list: ContactListResolver,
    },
  },
  {
    path: 'new',
    component: ContactDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Contacts',
    },
    resolve: {
      item: ContactResolver,
      departments: DepartmentListResolver,
    },
  },
  {
    path: ':id',
    component: ContactDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Contacts',
    },
    resolve: {
      item: ContactResolver,
      departments: DepartmentListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(advocatesRoutes)],
  exports: [RouterModule],
  providers: [ContactListResolver, ContactResolver],
})
export class ContactsRoutingModule {}
