import { Act } from './act';
import { CaseSource } from './case-source';
import { CaseType } from './case-type';
import { Court } from './court';
import { CourtStatus } from './court-status';
import { Department } from './department';
import { Hearing } from './hearing';
import { Nature } from './nature';
import { Office } from './office';
import { OfficeStatus } from './office-status';

export class Case {
  id: string | undefined;
  caseSource: CaseSource;
  officeFileNumber: string;
  courtCaseNumber: string;
  year: string;
  title: string;
  docketNumber: string;
  receiptDate: string | null;
  limitationDate: string | null;
  filingDate: string | null;
  appearOnBehalfOf: string;
  questionOfLaw: string;
  aorName: string;
  opposingCouncilAor: string;
  opposingCouncilDetail: string;
  lowerCourtCaseNumber: string;
  dateOfImpugnedJudgement: string | null;
  briefDescription: string;
  remarks: string;
  slpCounter: string;
  contactDetail: string;
  caseConnectedWith: string;
  bunchCases: string;
  court?: Court;
  department?: Department;
  office?: Office;
  caseType?: CaseType;
  act?: Act;
  nature?: Nature;
  officeStatus?: OfficeStatus;
  courtStatus?: CourtStatus;
  hearings: Hearing[];

  public constructor(init?: Partial<Case>) {
    this.id = undefined;
    this.caseSource = new CaseSource();
    this.officeFileNumber = '';
    this.courtCaseNumber = '';
    this.year = '';
    this.title = '';
    this.docketNumber = '';
    this.receiptDate = '';
    this.limitationDate = '';
    this.filingDate = '';
    this.appearOnBehalfOf = '';
    this.questionOfLaw = '';
    this.aorName = '';
    this.opposingCouncilAor = '';
    this.opposingCouncilDetail = '';
    this.lowerCourtCaseNumber = '';
    this.dateOfImpugnedJudgement = '';
    this.briefDescription = '';
    this.remarks = '';
    this.slpCounter = '';
    this.contactDetail = '';
    this.caseConnectedWith = '';
    this.bunchCases = '';
    this.hearings = [];
    Object.assign(this, init);
  }
}
