import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { CourtStatus } from '../core/court-status';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/court-statuses';
const serviceName = 'CourtStatusService';

@Injectable({
  providedIn: 'root',
})
export class CourtStatusService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<CourtStatus> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<CourtStatus>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`)),
      ) as Observable<CourtStatus>;
  }

  list(): Observable<CourtStatus[]> {
    return this.http
      .get<CourtStatus[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<CourtStatus[]>;
  }

  save(courtStatus: CourtStatus): Observable<CourtStatus> {
    return this.http
      .post<CourtStatus>(url, courtStatus, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<CourtStatus>;
  }

  update(courtStatus: CourtStatus): Observable<CourtStatus> {
    return this.http
      .put<CourtStatus>(`${url}/${courtStatus.id}`, courtStatus, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<CourtStatus>;
  }

  saveOrUpdate(courtStatus: CourtStatus): Observable<CourtStatus> {
    if (!courtStatus.id) {
      return this.save(courtStatus);
    }
    return this.update(courtStatus);
  }

  delete(id: string): Observable<CourtStatus> {
    return this.http
      .delete<CourtStatus>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<CourtStatus>;
  }
}
