import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Court } from '../core/court';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/courts';
const serviceName = 'CourtService';

@Injectable({
  providedIn: 'root',
})
export class CourtService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Court> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Court>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Court>;
  }

  list(): Observable<Court[]> {
    return this.http
      .get<Court[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Court[]>;
  }

  save(court: Court): Observable<Court> {
    return this.http
      .post<Court>(url, court, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Court>;
  }

  update(court: Court): Observable<Court> {
    return this.http
      .put<Court>(`${url}/${court.id}`, court, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Court>;
  }

  saveOrUpdate(court: Court): Observable<Court> {
    if (!court.id) {
      return this.save(court);
    }
    return this.update(court);
  }

  delete(id: string): Observable<Court> {
    return this.http
      .delete<Court>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Court>;
  }
}
