import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { Department } from '../core/department';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/departments';
const serviceName = 'DepartmentService';

@Injectable({
  providedIn: 'root',
})
export class DepartmentService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Department> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Department>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`)),
      ) as Observable<Department>;
  }

  list(): Observable<Department[]> {
    return this.http
      .get<Department[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Department[]>;
  }

  save(department: Department): Observable<Department> {
    return this.http
      .post<Department>(url, department, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Department>;
  }

  update(department: Department): Observable<Department> {
    return this.http
      .put<Department>(`${url}/${department.id}`, department, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Department>;
  }

  saveOrUpdate(department: Department): Observable<Department> {
    if (!department.id) {
      return this.save(department);
    }
    return this.update(department);
  }

  delete(id: string): Observable<Department> {
    return this.http
      .delete<Department>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Department>;
  }
}
