import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { Nature } from '../core/nature';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/natures';
const serviceName = 'NatureService';

@Injectable({
  providedIn: 'root',
})
export class NatureService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Nature> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Nature>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Nature>;
  }

  list(): Observable<Nature[]> {
    return this.http
      .get<Nature[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Nature[]>;
  }

  save(nature: Nature): Observable<Nature> {
    return this.http
      .post<Nature>(url, nature, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Nature>;
  }

  update(nature: Nature): Observable<Nature> {
    return this.http
      .put<Nature>(`${url}/${nature.id}`, nature, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Nature>;
  }

  saveOrUpdate(nature: Nature): Observable<Nature> {
    if (!nature.id) {
      return this.save(nature);
    }
    return this.update(nature);
  }

  delete(id: string): Observable<Nature> {
    return this.http
      .delete<Nature>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Nature>;
  }
}
