import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { OfficeStatus } from '../core/office-status';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/office-statuses';
const serviceName = 'OfficeStatusService';

@Injectable({
  providedIn: 'root',
})
export class OfficeStatusService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<OfficeStatus> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<OfficeStatus>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`)),
      ) as Observable<OfficeStatus>;
  }

  list(): Observable<OfficeStatus[]> {
    return this.http
      .get<OfficeStatus[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<OfficeStatus[]>;
  }

  save(officeStatus: OfficeStatus): Observable<OfficeStatus> {
    return this.http
      .post<OfficeStatus>(url, officeStatus, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<OfficeStatus>;
  }

  update(officeStatus: OfficeStatus): Observable<OfficeStatus> {
    return this.http
      .put<OfficeStatus>(`${url}/${officeStatus.id}`, officeStatus, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<OfficeStatus>;
  }

  saveOrUpdate(officeStatus: OfficeStatus): Observable<OfficeStatus> {
    if (!officeStatus.id) {
      return this.save(officeStatus);
    }
    return this.update(officeStatus);
  }

  delete(id: string): Observable<OfficeStatus> {
    return this.http
      .delete<OfficeStatus>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<OfficeStatus>;
  }
}
