import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { DepartmentListResolver } from '../departments/department-list-resolver.service';

import { OfficeDetailComponent } from './office-detail/office-detail.component';
import { OfficeListResolver } from './office-list-resolver.service';
import { OfficeListComponent } from './office-list/office-list.component';
import { OfficeResolver } from './office-resolver.service';

const advocatesRoutes: Routes = [
  {
    path: '',
    component: OfficeListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Offices',
    },
    resolve: {
      list: OfficeListResolver,
    },
  },
  {
    path: 'new',
    component: OfficeDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Offices',
    },
    resolve: {
      item: OfficeResolver,
      departments: DepartmentListResolver,
    },
  },
  {
    path: ':id',
    component: OfficeDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Offices',
    },
    resolve: {
      item: OfficeResolver,
      departments: DepartmentListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(advocatesRoutes)],
  exports: [RouterModule],
  providers: [OfficeListResolver, OfficeResolver],
})
export class OfficesRoutingModule {}
