import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { AbstractControl, FormArray, FormBuilder, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';

import { ToasterService } from '../../core/toaster.service';
import { User } from '../../core/user';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { UserService } from '../user.service';

@Component({
  selector: 'app-user-detail',
  templateUrl: './user-detail.component.html',
  styleUrls: ['./user-detail.component.css'],
})
export class UserDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup;
  item: User = new User();
  hide: boolean;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
    private toaster: ToasterService,
    private dialog: MatDialog,
    private ser: UserService,
  ) {
    this.hide = true;
    // Create form
    this.form = this.fb.group({
      name: '',
      password: '',
      lockedOut: '',
      roles: this.fb.array([]),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: User };
      this.showItem(data.item);
    });
  }

  showItem(item: User) {
    this.item = item;
    (this.form.get('name') as AbstractControl).setValue(item.name);
    (this.form.get('password') as AbstractControl).setValue('');
    (this.form.get('lockedOut') as AbstractControl).setValue(item.lockedOut);
    this.form.setControl(
      'roles',
      this.fb.array(
        item.roles.map((x) =>
          this.fb.group({
            role: x.enabled,
          }),
        ),
      ),
    );
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        if (this.nameElement !== undefined) {
          this.nameElement.nativeElement.focus();
        }
      }
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/users');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/users');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete User?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): User {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.password = formModel.password;
    this.item.lockedOut = formModel.lockedOut;
    const array = this.form.get('roles') as FormArray;
    if (this.item.roles !== undefined) {
      this.item.roles.forEach((item, index) => {
        item.enabled = array.controls[index].value.role;
      });
    }
    return this.item;
  }
}
